//+------------------------------------------------------------------+
//|                                                       Series.mqh |
//|                                    Copyright (c) 2020, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//+------------------------------------------------------------------+

#ifndef EXTENDED_FUNCTORS
  #define EXTENDED_FUNCTORS
#endif

#include "../Functors.mqh"


enum SERIES_FUNC_TYPE
{
  TIME,
  OPEN,
  HIGH,
  LOW,
  CLOSE,
  VOLUME,
  REALVOLUME,
  SPREAD,
};

class SeriesFunc: public BaseFunc
{
  private:
    static BaseFuncFactory factory;
    
  protected:
    const SERIES_FUNC_TYPE type;
    const string symbol;
    const ENUM_TIMEFRAMES timeframe;
    
    static BaseFunc *legend(void)
    {
      string msg = "Attached Series funcs:";
      for(int i = SERIES_FUNC_TYPE::TIME; i <= SERIES_FUNC_TYPE::SPREAD; i++)
      {
        msg += EnumToString((SERIES_FUNC_TYPE)i) + ";";
      }
      Print(msg);
      return NULL;
    }
    
  public:
    SeriesFunc(const string n, const int t, const string s, const ENUM_TIMEFRAMES p): BaseFunc(n), type((SERIES_FUNC_TYPE)t), symbol(s), timeframe(p)
    {
    }
    
    static BaseFunc *create(const string name, const string s = NULL, const ENUM_TIMEFRAMES tf = 0)
    {
      if(name == NULL)
      {
        return legend();
      }

      for(int i = SERIES_FUNC_TYPE::TIME; i <= SERIES_FUNC_TYPE::SPREAD; i++)
      {
        if(name == EnumToString((SERIES_FUNC_TYPE)i))
        {
          return new SeriesFunc(name, i, s, tf);
        }
      }
      return NULL;
    }
    
    virtual double execute(const double &params[]) override
    {
      const int bar = (int)params[0];
      switch(type)
      {
        case TIME: return (double)iTime(symbol, timeframe, bar);
        case OPEN: return iOpen(symbol, timeframe, bar);
        case HIGH: return iHigh(symbol, timeframe, bar);
        case LOW:  return iLow(symbol, timeframe, bar);
        case CLOSE: return iClose(symbol, timeframe, bar);
        case VOLUME: return (double)iVolume(symbol, timeframe, bar);
        case REALVOLUME: return (double)iRealVolume(symbol, timeframe, bar);
        case SPREAD: return (double)iSpread(symbol, timeframe, bar);
      }
      return nan;
    }

    virtual bool hasContext(string &s, ENUM_TIMEFRAMES &t) override
    {
      s = symbol;
      t = timeframe;
      return true;
    }
};

static BaseFuncFactory SeriesFunc::factory(SeriesFunc::create);
